#!/usr/bin/env python3.9
# -*- coding: utf-8 -*-

from typing import Generator, Dict, Any, List, Iterator

from loguru import logger


class ScrollSource:
    """

    Sample:
        _source = ScrollSource(es_search_info, es_client)
        # Delete scroll_id by using:
        _source.delete_scroll_id()

    """

    def __init__(self, search_info: Dict[str, any], es_client) -> None:
        self.scroll_id = None
        self.client = es_client
        self.search_info = search_info

    def __iter__(self) -> Generator[List[Dict[Any, Any]], None, None]:
        """Generator fn to get the scroll _source"""
        total_size = 0
        data = self.client.search(**self.search_info)
        scroll_size = len(data['hits']['hits'])
        self.scroll_id = data['_scroll_id']
        logger.info(f"Scrolling {str(self.client)} by index: {self.search_info['index']}\n scroll_id: {self.scroll_id}")
        if scroll_size == 0:
            self.client.clear_scroll(scroll_id=self.scroll_id)
        while scroll_size > 0:
            request_list = data['hits']['hits']
            total_size += scroll_size
            logger.info(f"scroll size: {total_size}")
            data = self.client.scroll(scroll_id=self.scroll_id, scroll=self.search_info["scroll"])
            scroll_size = len(data['hits']['hits'])
            yield request_list

    def delete_scroll_id(self):
        if self.scroll_id:
            self.client.clear_scroll(scroll_id=self.scroll_id)



#!/usr/bin/env python3.9
# -*- coding: utf-8 -*-
import csv
from typing import Dict, Any, List

from scroll_source import ScrollSource
from elasticsearch import Elasticsearch, helpers


class ElasticSearchCollections:

    def __init__(self, hosts: Any = ..., **kwargs: Any) -> None:
        self.scroll_id = None
        self.client = Elasticsearch(hosts, **kwargs)

    def bulk(self, actions: List[Dict[Any, Any]]):
        helpers.bulk(client=self.client, actions=actions)

    def upload_source_from_csv(self, csvfile):
        """
        update $.'_source' from csvfile.
        fn need to be re-written once the date parse logic changed.
        """
        with open(csvfile, "r") as f:
            csv_reader = csv.DictReader(f)
            actions = []
            for row in csv_reader:
                actions.append(row)
        if actions:
            self.bulk(actions)

    def download_source_to_csv(self, csvfile: str, field_names: List[str], search_info: Dict[str, any]):
        """
        Parse the search result ($.'_source': dict) to csv.
        Example: download_source_to_csv("test1.csv", ['_source']['xxxData'], es_search_info)
        fn need to be re-written once the date parse logic changed.
        """
        with open(csvfile, "w") as f:
            csv_writer = csv.DictWriter(f, fieldnames=field_names)
            csv_writer.writeheader()
            _source = ScrollSource(search_info, self.client)

            for _data_list in _source:
                for i in _data_list:
                    csv_writer.writerow(i['_source'])

            _source.delete_scroll_id()



import setuptools

try:
    from setuptools import setup
except ImportError:
    from distutils.core import setup

with open("README.md", "r") as fh:
    long_description = fh.read()

requires = ["loguru", "elasticsearch == 7.11.0"]

setup(name="elasticSearch_collections",
      description="Collections of ElasticSearch pyscripts for human",
      long_description=long_description,
      long_description_content_type="text/markdown",
      license="MIT",
      version="1.1",
      author="Alex Ng",
      author_email="alex_q_wu@qq.com",
      maintainer="Alex Ng",
      maintainer_email="alex_q_wu@qq.com",
      url="https://github.com/AlexNg9527/ElasticSearchCollections",
      packages=setuptools.find_packages(),
      install_requires=requires,
      classifiers=[
          'Programming Language :: Python :: 3',
      ])
